@@ -24,6 +24,8 @@
 #include <isc/print.h>
 #include <isc/queue.h>
 #include <isc/random.h>
+#include <isc/safe.h>
+#include <isc/serial.h>
 #include <isc/stats.h>
 #include <isc/stdio.h>
 #include <isc/string.h>
@@ -31,6 +33,12 @@
 #include <isc/timer.h>
 #include <isc/util.h>
 
+#ifdef AES_SIT
+#include <isc/aes.h>
+#else
+#include <isc/hmacsha.h>
+#endif
+
 #include <dns/db.h>
 #include <dns/dispatch.h>
 #include <dns/events.h>
@@ -112,6 +120,10 @@
  */
 #endif
 
+#define COOKIE_SIZE 24U /* 8 + 4 + 4 + 8 */
+
+#define WANTNSID(x) (((x)->attributes & NS_CLIENTATTR_WANTNSID) != 0)
+#define WANTEXPIRE(x) (((x)->attributes & NS_CLIENTATTR_WANTEXPIRE) != 0)
 
 /*% nameserver client manager structure */
 struct ns_clientmgr {
@@ -235,6 +247,10 @@ static isc_result_t get_client(ns_clientmgr_t *manager, ns_interface_t *ifp,
 			       dns_dispatch_t *disp, isc_boolean_t tcp);
 static inline isc_boolean_t
 allowed(isc_netaddr_t *addr, dns_name_t *signer, dns_acl_t *acl);
+#ifdef ISC_PLATFORM_USESIT
+static void compute_cookie(ns_client_t *client, isc_uint32_t when,
+			isc_uint32_t nonce, isc_buffer_t *buf);
+#endif
 
 void
 ns_client_recursing(ns_client_t *client) {
@@ -548,6 +564,8 @@ exit_check(ns_client_t *client) {
 		isc_event_free((isc_event_t **)&client->sendevent);
 		isc_event_free((isc_event_t **)&client->recvevent);
 		isc_timer_detach(&client->timer);
+		if (client->delaytimer != NULL)
+			isc_timer_detach(&client->delaytimer);
 
 		if (client->tcpbuf != NULL)
 			isc_mem_put(client->mctx, client->tcpbuf,
@@ -686,8 +704,11 @@ ns_client_endrequest(ns_client_t *client) {
 	client->ednsversion = -1;
 	dns_message_reset(client->message, DNS_MESSAGE_INTENTPARSE);
 
-	if (client->recursionquota != NULL)
+	if (client->recursionquota != NULL) {
 		isc_quota_detach(&client->recursionquota);
+		isc_stats_decrement(ns_g_server->nsstats,
+				    dns_nsstatscounter_recursclients);
+	}
 
 	/*
 	 * Clear all client attributes that are specific to
@@ -805,10 +826,24 @@ client_allocsendbuf(ns_client_t *client, isc_buffer_t *buffer,
 		}
 	} else {
 		data = sendbuf;
+#ifdef ISC_PLATFORM_USESIT
+		if ((client->attributes & NS_CLIENTATTR_HAVESIT) == 0) {
+			if (client->view != NULL)
+				bufsize = client->view->situdp;
+			else
+				bufsize = 512;
+		} else
+			bufsize = client->udpsize;
+		if (bufsize > client->udpsize)
+			bufsize = client->udpsize;
+		if (bufsize > SEND_BUFFER_SIZE)
+			bufsize = SEND_BUFFER_SIZE;
+#else
 		if (client->udpsize < SEND_BUFFER_SIZE)
 			bufsize = client->udpsize;
 		else
 			bufsize = SEND_BUFFER_SIZE;
+#endif
 		if (length > bufsize) {
 			result = ISC_R_NOSPACE;
 			goto done;
@@ -832,6 +867,7 @@ client_sendpkg(ns_client_t *client, isc_buffer_t *buffer) {
 	isc_netaddr_t netaddr;
 	int match;
 	unsigned int sockflags = ISC_SOCKFLAG_IMMEDIATE;
+	isc_dscp_t dispdscp = -1;
 
 	if (TCP_CLIENT(client)) {
 		sock = client->tcpsocket;
@@ -857,6 +893,20 @@ client_sendpkg(ns_client_t *client, isc_buffer_t *buffer) {
 	else
 		pktinfo = NULL;
 
+	if (client->dispatch != NULL) {
+		dispdscp = dns_dispatch_getdscp(client->dispatch);
+		if (dispdscp != -1)
+			client->dscp = dispdscp;
+	}
+
+	if (client->dscp == -1) {
+		client->sendevent->attributes &= ~ISC_SOCKEVENTATTR_DSCP;
+		client->sendevent->dscp = 0;
+	} else {
+		client->sendevent->attributes |= ISC_SOCKEVENTATTR_DSCP;
+		client->sendevent->dscp = client->dscp;
+	}
+
 	isc_buffer_usedregion(buffer, &r);
 
 	/*
@@ -865,6 +915,7 @@ client_sendpkg(ns_client_t *client, isc_buffer_t *buffer) {
 	 * MTU link force fragmentation at 1280 if it is a IPv6
 	 * response.
 	 */
+	client->sendevent->attributes &= ~ISC_SOCKEVENTATTR_USEMINMTU;
 	if (!TCP_CLIENT(client) && r.length > 1432)
 		client->sendevent->attributes |= ISC_SOCKEVENTATTR_USEMINMTU;
 
@@ -929,8 +980,8 @@ ns_client_sendraw(ns_client_t *client, dns_message_t *message) {
 	ns_client_next(client, result);
 }
 
-void
-ns_client_send(ns_client_t *client) {
+static void
+client_send(ns_client_t *client) {
 	isc_result_t result;
 	unsigned char *data;
 	isc_buffer_t buffer;
@@ -970,10 +1021,11 @@ ns_client_send(ns_client_t *client) {
 			preferred_glue = DNS_MESSAGERENDER_PREFER_AAAA;
 	}
 
-#ifdef ALLOW_FILTER_AAAA_ON_V4
+#ifdef ALLOW_FILTER_AAAA
 	/*
 	 * filter-aaaa-on-v4 yes or break-dnssec option to suppress
-	 * AAAA records
+	 * AAAA records.
+	 *
 	 * We already know that request came via IPv4,
 	 * that we have both AAAA and A records,
 	 * and that we either have no signatures that the client wants
@@ -988,6 +1040,16 @@ ns_client_send(ns_client_t *client) {
 	}
 #endif
 
+	/*
+	 * Create an OPT for our reply.
+	 */
+	if ((client->attributes & NS_CLIENTATTR_WANTOPT) != 0) {
+		result = ns_client_addopt(client, client->message,
+					  &client->opt);
+		if (result != ISC_R_SUCCESS)
+			goto done;
+	}
+
 	/*
 	 * XXXRTH  The following doesn't deal with TCP buffer resizing.
 	 */
@@ -1033,13 +1095,11 @@ ns_client_send(ns_client_t *client) {
 	}
 	if (result != ISC_R_SUCCESS)
 		goto done;
-#ifdef USE_RRL
 	/*
 	 * Stop after the question if TC was set for rate limiting.
 	 */
 	if ((client->message->flags & DNS_MESSAGEFLAG_TC) != 0)
 		goto renderend;
-#endif /* USE_RRL */
 	result = dns_message_rendersection(client->message,
 					   DNS_SECTION_ANSWER,
 					   DNS_MESSAGERENDER_PARTIAL |
@@ -1117,6 +1177,72 @@ ns_client_send(ns_client_t *client) {
 	ns_client_next(client, result);
 }
 
+/*
+ * Completes the sending of a delayed client response.
+ */
+static void
+client_delay(isc_task_t *task, isc_event_t *event) {
+	ns_client_t *client;
+
+	REQUIRE(event != NULL);
+	REQUIRE(event->ev_type == ISC_TIMEREVENT_LIFE ||
+		event->ev_type == ISC_TIMEREVENT_IDLE);
+	client = event->ev_arg;
+	REQUIRE(NS_CLIENT_VALID(client));
+	REQUIRE(task == client->task);
+	REQUIRE(client->delaytimer != NULL);
+
+	UNUSED(task);
+
+	CTRACE("client_delay");
+
+	isc_event_free(&event);
+	isc_timer_detach(&client->delaytimer);
+
+	client_send(client);
+	ns_client_detach(&client);
+}
+
+void
+ns_client_send(ns_client_t *client) {
+
+	/*
+	 * Delay the response by ns_g_delay ms.
+	 */
+	if (ns_g_delay != 0) {
+		ns_client_t *dummy = NULL;
+		isc_result_t result;
+		isc_interval_t interval;
+
+		/*
+		 * Replace ourselves if we have not already been replaced.
+		 */
+		if (!client->mortal) {
+			result = ns_client_replace(client);
+			if (result != ISC_R_SUCCESS)
+				goto nodelay;
+		}
+
+		ns_client_attach(client, &dummy);
+		if (ns_g_delay >= 1000)
+			isc_interval_set(&interval, ns_g_delay / 1000,
+					 (ns_g_delay % 1000) * 1000000);
+		else
+			isc_interval_set(&interval, 0, ns_g_delay * 1000000);
+		result = isc_timer_create(client->manager->timermgr,
+					  isc_timertype_once, NULL, &interval,
+					  client->task, client_delay,
+					  client, &client->delaytimer);
+		if (result == ISC_R_SUCCESS)
+			return;
+
+		ns_client_detach(&dummy);
+	}
+
+ nodelay:
+	client_send(client);
+}
+
 #if NS_CLIENT_DROPPORT
 #define DROPPORT_NO		0
 #define DROPPORT_REQUEST	1
@@ -1179,7 +1305,6 @@ ns_client_error(ns_client_t *client, isc_result_t result) {
 	}
 #endif
 
-#ifdef USE_RRL
 	/*
 	 * Try to rate limit error responses.
 	 */
@@ -1229,7 +1354,6 @@ ns_client_error(ns_client_t *client, isc_result_t result) {
 			}
 		}
 	}
-#endif /* USE_RRL */
 
 	/*
 	 * Message may be an in-progress reply that we had trouble
@@ -1285,9 +1409,14 @@ ns_client_error(ns_client_t *client, isc_result_t result) {
 	ns_client_send(client);
 }
 
-static inline isc_result_t
-client_addopt(ns_client_t *client) {
+isc_result_t
+ns_client_addopt(ns_client_t *client, dns_message_t *message,
+		 dns_rdataset_t **opt)
+{
 	char nsid[BUFSIZ], *nsidp;
+#ifdef ISC_PLATFORM_USESIT
+	unsigned char cookie[COOKIE_SIZE];
+#endif
 	isc_result_t result;
 	dns_view_t *view;
 	dns_resolver_t *resolver;
@@ -1295,8 +1424,11 @@ client_addopt(ns_client_t *client) {
 	dns_ednsopt_t ednsopts[DNS_EDNSOPTIONS];
 	int count = 0;
 	unsigned int flags;
+	unsigned char expire[4];
 
-	REQUIRE(client->opt == NULL);	/* XXXRTH free old. */
+	REQUIRE(NS_CLIENT_VALID(client));
+	REQUIRE(opt != NULL && *opt == NULL);
+	REQUIRE(message != NULL);
 
 	view = client->view;
 	resolver = (view != NULL) ? view->resolver : NULL;
@@ -1308,7 +1440,7 @@ client_addopt(ns_client_t *client) {
 	flags = client->extflags & DNS_MESSAGEEXTFLAG_REPLYPRESERVE;
 
 	/* Set EDNS options if applicable */
-	if ((client->attributes & NS_CLIENTATTR_WANTNSID) != 0 &&
+	if (WANTNSID(client) &&
 	    (ns_g_server->server_id != NULL ||
 	     ns_g_server->server_usehostname)) {
 		if (ns_g_server->server_usehostname) {
@@ -1327,8 +1459,40 @@ client_addopt(ns_client_t *client) {
 		count++;
 	}
  no_nsid:
-	result = dns_message_buildopt(client->message, &client->opt, 0,
-				      udpsize, flags, ednsopts, count);
+#ifdef ISC_PLATFORM_USESIT
+	if ((client->attributes & NS_CLIENTATTR_WANTSIT) != 0) {
+		isc_buffer_t buf;
+		isc_stdtime_t now;
+		isc_uint32_t nonce;
+
+		isc_buffer_init(&buf, cookie, sizeof(cookie));
+		isc_stdtime_get(&now);
+		isc_random_get(&nonce);
+
+		compute_cookie(client, now, nonce, &buf);
+
+		INSIST(count < DNS_EDNSOPTIONS);
+		ednsopts[count].code = DNS_OPT_COOKIE;
+		ednsopts[count].length = COOKIE_SIZE;
+		ednsopts[count].value = cookie;
+		count++;
+	}
+#endif
+	if ((client->attributes & NS_CLIENTATTR_HAVEEXPIRE) != 0) {
+		isc_buffer_t buf;
+
+		INSIST(count < DNS_EDNSOPTIONS);
+
+		isc_buffer_init(&buf, expire, sizeof(expire));
+		isc_buffer_putuint32(&buf, client->expire);
+		ednsopts[count].code = DNS_OPT_EXPIRE;
+		ednsopts[count].length = 4;
+		ednsopts[count].value = expire;
+		count++;
+	}
+
+	result = dns_message_buildopt(message, opt, 0, udpsize, flags,
+				      ednsopts, count);
 	return (result);
 }
 
@@ -1410,6 +1574,187 @@ ns_client_isself(dns_view_t *myview, dns_tsigkey_t *mykey,
 	return (ISC_TF(view == myview));
 }
 
+#ifdef ISC_PLATFORM_USESIT
+static void
+compute_cookie(ns_client_t *client, isc_uint32_t when, isc_uint32_t nonce,
+	       isc_buffer_t *buf)
+{
+#ifdef AES_SIT
+	unsigned char digest[ISC_AES_BLOCK_LENGTH];
+	unsigned char input[4 + 4 + 16];
+	isc_netaddr_t netaddr;
+	unsigned char *cp;
+	unsigned int i;
+
+	memset(input, 0, sizeof(input));
+	cp = isc_buffer_used(buf);
+	isc_buffer_putmem(buf, client->cookie, 8);
+	isc_buffer_putuint32(buf, nonce);
+	isc_buffer_putuint32(buf, when);
+	memmove(input, cp, 16);
+	isc_aes128_crypt(ns_g_server->secret, input, digest);
+	for (i = 0; i < 8; i++)
+		input[i] = digest[i] ^ digest[i + 8];
+	isc_netaddr_fromsockaddr(&netaddr, &client->peeraddr);
+	switch (netaddr.family) {
+	case AF_INET:
+		memmove(input + 8, (unsigned char *)&netaddr.type.in, 4);
+		memset(input + 12, 0, 4);
+		isc_aes128_crypt(ns_g_server->secret, input, digest);
+		break;
+	case AF_INET6:
+		memmove(input + 8, (unsigned char *)&netaddr.type.in6, 16);
+		isc_aes128_crypt(ns_g_server->secret, input, digest);
+		for (i = 0; i < 8; i++)
+			input[i + 8] = digest[i] ^ digest[i + 8];
+		isc_aes128_crypt(ns_g_server->secret, input + 8, digest);
+		break;
+	}
+	for (i = 0; i < 8; i++)
+		digest[i] ^= digest[i + 8];
+	isc_buffer_putmem(buf, digest, 8);
+#endif
+#ifdef HMAC_SHA1_SIT
+	unsigned char digest[ISC_SHA1_DIGESTLENGTH];
+	isc_netaddr_t netaddr;
+	unsigned char *cp;
+	isc_hmacsha1_t hmacsha1;
+
+	cp = isc_buffer_used(buf);
+	isc_buffer_putmem(buf, client->cookie, 8);
+	isc_buffer_putuint32(buf, nonce);
+	isc_buffer_putuint32(buf, when);
+
+	isc_hmacsha1_init(&hmacsha1,
+			  ns_g_server->secret,
+			  ISC_SHA1_DIGESTLENGTH);
+	isc_hmacsha1_update(&hmacsha1, cp, 16);
+	isc_netaddr_fromsockaddr(&netaddr, &client->peeraddr);
+	switch (netaddr.family) {
+	case AF_INET:
+		isc_hmacsha1_update(&hmacsha1,
+				    (unsigned char *)&netaddr.type.in, 4);
+		break;
+	case AF_INET6:
+		isc_hmacsha1_update(&hmacsha1,
+				    (unsigned char *)&netaddr.type.in6, 16);
+		break;
+	}
+	isc_hmacsha1_update(&hmacsha1, client->cookie, sizeof(client->cookie));
+	isc_hmacsha1_sign(&hmacsha1, digest, sizeof(digest));
+	isc_buffer_putmem(buf, digest, 8);
+	isc_hmacsha1_invalidate(&hmacsha1);
+#endif
+#ifdef HMAC_SHA256_SIT
+	unsigned char digest[ISC_SHA256_DIGESTLENGTH];
+	isc_netaddr_t netaddr;
+	unsigned char *cp;
+	isc_hmacsha256_t hmacsha256;
+
+	cp = isc_buffer_used(buf);
+	isc_buffer_putmem(buf, client->cookie, 8);
+	isc_buffer_putuint32(buf, nonce);
+	isc_buffer_putuint32(buf, when);
+
+	isc_hmacsha256_init(&hmacsha256,
+			    ns_g_server->secret,
+			    ISC_SHA256_DIGESTLENGTH);
+	isc_hmacsha256_update(&hmacsha256, cp, 16);
+	isc_netaddr_fromsockaddr(&netaddr, &client->peeraddr);
+	switch (netaddr.family) {
+	case AF_INET:
+		isc_hmacsha256_update(&hmacsha256,
+				      (unsigned char *)&netaddr.type.in, 4);
+		break;
+	case AF_INET6:
+		isc_hmacsha256_update(&hmacsha256,
+				      (unsigned char *)&netaddr.type.in6, 16);
+		break;
+	}
+	isc_hmacsha256_update(&hmacsha256, client->cookie,
+			      sizeof(client->cookie));
+	isc_hmacsha256_sign(&hmacsha256, digest, sizeof(digest));
+	isc_buffer_putmem(buf, digest, 8);
+	isc_hmacsha256_invalidate(&hmacsha256);
+#endif
+}
+
+static void
+process_cookie(ns_client_t *client, isc_buffer_t *buf, size_t optlen) {
+	unsigned char dbuf[COOKIE_SIZE];
+	unsigned char *old;
+	isc_stdtime_t now;
+	isc_uint32_t when;
+	isc_uint32_t nonce;
+	isc_buffer_t db;
+
+	/*
+	 * If we have already seen a SIT option skip this SIT option.
+	 */
+	if ((client->attributes & NS_CLIENTATTR_WANTSIT) != 0) {
+		isc_buffer_forward(buf, (isc_uint32_t)optlen);
+		return;
+	}
+	client->attributes |= NS_CLIENTATTR_WANTSIT;
+
+	isc_stats_increment(ns_g_server->nsstats,
+			    dns_nsstatscounter_sitopt);
+
+	if (optlen != COOKIE_SIZE) {
+		/*
+		 * Not our token.
+		 */
+		if (optlen >= 8U)
+			memmove(client->cookie, isc_buffer_current(buf), 8);
+		else
+			memset(client->cookie, 0, 8);
+		isc_buffer_forward(buf, (unsigned int)optlen);
+
+		if (optlen == 8U)
+			isc_stats_increment(ns_g_server->nsstats,
+					    dns_nsstatscounter_sitnew);
+		else
+			isc_stats_increment(ns_g_server->nsstats,
+					    dns_nsstatscounter_sitbadsize);
+		return;
+	}
+
+	/*
+	 * Process all of the incoming buffer.
+	 */
+	old = isc_buffer_current(buf);
+	memmove(client->cookie, old, 8);
+	isc_buffer_forward(buf, 8);
+	nonce = isc_buffer_getuint32(buf);
+	when = isc_buffer_getuint32(buf);
+	isc_buffer_forward(buf, 8);
+
+	/*
+	 * Allow for a 5 minute clock skew between servers sharing a secret.
+	 * Only accept COOKIE if we have talked to the client in the last hour.
+	 */
+	isc_stdtime_get(&now);
+	if (isc_serial_gt(when, (now + 300)) ||		/* In the future. */
+	    isc_serial_lt(when, (now - 3600))) {	/* In the past. */
+		isc_stats_increment(ns_g_server->nsstats,
+				    dns_nsstatscounter_sitbadtime);
+		return;
+	}
+
+	isc_buffer_init(&db, dbuf, sizeof(dbuf));
+	compute_cookie(client, when, nonce, &db);
+
+	if (!isc_safe_memequal(old, dbuf, COOKIE_SIZE)) {
+		isc_stats_increment(ns_g_server->nsstats,
+				    dns_nsstatscounter_sitnomatch);
+		return;
+	}
+	isc_stats_increment(ns_g_server->nsstats,
+			    dns_nsstatscounter_sitmatch);
+	client->attributes |= NS_CLIENTATTR_HAVESIT;
+}
+#endif
+
 static isc_result_t
 process_opt(ns_client_t *client, dns_rdataset_t *opt) {
 	dns_rdata_t rdata;
@@ -1444,7 +1789,8 @@ process_opt(ns_client_t *client, dns_rdataset_t *opt) {
 	if (client->ednsversion > 0) {
 		isc_stats_increment(ns_g_server->nsstats,
 				    dns_nsstatscounter_badednsver);
-		result = client_addopt(client);
+		result = ns_client_addopt(client, client->message,
+					  &client->opt);
 		if (result == ISC_R_SUCCESS)
 			result = DNS_R_BADVERS;
 		ns_client_error(client, result);
@@ -1463,10 +1809,29 @@ process_opt(ns_client_t *client, dns_rdataset_t *opt) {
 			optlen = isc_buffer_getuint16(&optbuf);
 			switch (optcode) {
 			case DNS_OPT_NSID:
+				if (!WANTNSID(client))
+					isc_stats_increment(
+						    ns_g_server->nsstats,
+						    dns_nsstatscounter_nsidopt);
 				client->attributes |= NS_CLIENTATTR_WANTNSID;
 				isc_buffer_forward(&optbuf, optlen);
 				break;
+#ifdef ISC_PLATFORM_USESIT
+			case DNS_OPT_COOKIE:
+				process_cookie(client, &optbuf, optlen);
+				break;
+#endif
+			case DNS_OPT_EXPIRE:
+				if (!WANTEXPIRE(client))
+					isc_stats_increment(
+						  ns_g_server->nsstats,
+						  dns_nsstatscounter_expireopt);
+				client->attributes |= NS_CLIENTATTR_WANTEXPIRE;
+				isc_buffer_forward(&optbuf, optlen);
+				break;
 			default:
+				isc_stats_increment(ns_g_server->nsstats,
+						  dns_nsstatscounter_otheropt);
 				isc_buffer_forward(&optbuf, optlen);
 				break;
 			}
@@ -1474,15 +1839,8 @@ process_opt(ns_client_t *client, dns_rdataset_t *opt) {
 	}
 
 	isc_stats_increment(ns_g_server->nsstats, dns_nsstatscounter_edns0in);
+	client->attributes |= NS_CLIENTATTR_WANTOPT;
 
-	/*
-	 * Create an OPT for our reply.
-	 */
-	result = client_addopt(client);
-	if (result != ISC_R_SUCCESS) {
-		ns_client_error(client, result);
-		goto cleanup;
-	}
  cleanup:
 	return (result);
 }
@@ -1534,6 +1892,13 @@ client_request(isc_task_t *task, isc_event_t *event) {
 			client->peeraddr = sevent->address;
 			client->peeraddr_valid = ISC_TRUE;
 		}
+		if ((sevent->attributes & ISC_SOCKEVENTATTR_DSCP) != 0) {
+			ns_client_log(client, NS_LOGCATEGORY_CLIENT,
+			      NS_LOGMODULE_CLIENT, ISC_LOG_DEBUG(90),
+			      "received DSCP %d", sevent->dscp);
+			if (client->dscp == -1)
+				client->dscp = sevent->dscp;
+		}
 		if ((sevent->attributes & ISC_SOCKEVENTATTR_PKTINFO) != 0) {
 			client->attributes |= NS_CLIENTATTR_PKTINFO;
 			client->pktinfo = sevent->pktinfo;
@@ -1679,7 +2044,8 @@ client_request(isc_task_t *task, isc_event_t *event) {
 		 * (typically FORMERR or SERVFAIL).
 		 */
 		if (result == DNS_R_OPTERR)
-			(void)client_addopt(client);
+			(void)ns_client_addopt(client, client->message,
+					       &client->opt);
 
 		ns_client_log(client, NS_LOGCATEGORY_CLIENT,
 			      NS_LOGMODULE_CLIENT, ISC_LOG_DEBUG(1),
@@ -2136,6 +2502,8 @@ client_create(ns_clientmgr_t *manager, ns_client_t **clientp) {
 		goto cleanup_task;
 	client->timerset = ISC_FALSE;
 
+	client->delaytimer = NULL;
+
 	client->message = NULL;
 	result = dns_message_create(client->mctx, DNS_MESSAGE_INTENTPARSE,
 				    &client->message);
@@ -2144,11 +2512,9 @@ client_create(ns_clientmgr_t *manager, ns_client_t **clientp) {
 
 	/* XXXRTH  Hardwired constants */
 
-	client->sendevent = (isc_socketevent_t *)
-			    isc_event_allocate(client->mctx, client,
-					       ISC_SOCKEVENT_SENDDONE,
-					       client_senddone, client,
-					       sizeof(isc_socketevent_t));
+	client->sendevent = isc_socket_socketevent(client->mctx, client,
+						   ISC_SOCKEVENT_SENDDONE,
+						   client_senddone, client);
 	if (client->sendevent == NULL) {
 		result = ISC_R_NOMEMORY;
 		goto cleanup_message;
@@ -2160,11 +2526,9 @@ client_create(ns_clientmgr_t *manager, ns_client_t **clientp) {
 		goto cleanup_sendevent;
 	}
 
-	client->recvevent = (isc_socketevent_t *)
-			    isc_event_allocate(client->mctx, client,
-					       ISC_SOCKEVENT_RECVDONE,
-					       client_request, client,
-					       sizeof(isc_socketevent_t));
+	client->recvevent = isc_socket_socketevent(client->mctx, client,
+						   ISC_SOCKEVENT_RECVDONE,
+						   client_request, client);
 	if (client->recvevent == NULL) {
 		result = ISC_R_NOMEMORY;
 		goto cleanup_recvbuf;
@@ -2191,6 +2555,7 @@ client_create(ns_clientmgr_t *manager, ns_client_t **clientp) {
 	client->tcpbuf = NULL;
 	client->opt = NULL;
 	client->udpsize = 512;
+	client->dscp = -1;
 	client->extflags = 0;
 	client->ednsversion = -1;
 	client->next = NULL;
@@ -2203,8 +2568,8 @@ client_create(ns_clientmgr_t *manager, ns_client_t **clientp) {
 	client->recursionquota = NULL;
 	client->interface = NULL;
 	client->peeraddr_valid = ISC_FALSE;
-#ifdef ALLOW_FILTER_AAAA_ON_V4
-	client->filter_aaaa = dns_v4_aaaa_ok;
+#ifdef ALLOW_FILTER_AAAA
+	client->filter_aaaa = dns_aaaa_ok;
 #endif
 	client->needshutdown = ns_g_clienttest;
 
@@ -2685,6 +3050,8 @@ get_client(ns_clientmgr_t *manager, ns_interface_t *ifp,
 	client->state = NS_CLIENTSTATE_READY;
 	INSIST(client->recursionquota == NULL);
 
+	client->dscp = ifp->dscp;
+
 	if (tcp) {
 		client->attributes |= NS_CLIENTATTR_TCP;
 		isc_socket_attach(ifp->tcpsocket,
