@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2015  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2011-2016  Internet Systems Consortium, Inc. ("ISC")
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -14,8 +14,6 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id$ */
-
 /* ! \file */
 
 #include <config.h>
@@ -25,14 +23,19 @@
 #include <stdio.h>
 #include <string.h>
 
+#include <isc/hash.h>
+
 #include <isc/crc64.h>
 #include <isc/hmacmd5.h>
 #include <isc/hmacsha.h>
 #include <isc/md5.h>
 #include <isc/sha1.h>
 #include <isc/util.h>
+#include <isc/print.h>
 #include <isc/string.h>
 
+#include <pk11/site.h>
+
 /*
  * Test data from RFC6234
  */
@@ -40,7 +43,7 @@
 unsigned char digest[ISC_SHA512_DIGESTLENGTH];
 unsigned char buffer[1024];
 const char *s;
-char str[2 * ISC_SHA512_DIGESTLENGTH + 1];
+char str[2 * ISC_SHA512_DIGESTLENGTH + 3];
 unsigned char key[20];
 
 isc_result_t
@@ -775,6 +778,7 @@ ATF_TC_BODY(isc_sha512, tc) {
 	}
 }
 
+#ifndef PK11_MD5_DISABLE
 ATF_TC(isc_md5);
 ATF_TC_HEAD(isc_md5, tc) {
 	atf_tc_set_md_var(tc, "descr", "md5 example from RFC1321");
@@ -847,6 +851,7 @@ ATF_TC_BODY(isc_md5, tc) {
 		testcase++;
 	}
 }
+#endif
 
 /* HMAC-SHA1 test */
 ATF_TC(isc_hmacsha1);
@@ -1643,6 +1648,7 @@ ATF_TC_BODY(isc_hmacsha512, tc) {
 }
 
 
+#ifndef PK11_MD5_DISABLE
 /* HMAC-MD5 Test */
 ATF_TC(isc_hmacmd5);
 ATF_TC_HEAD(isc_hmacmd5, tc) {
@@ -1784,6 +1790,7 @@ ATF_TC_BODY(isc_hmacmd5, tc) {
 		test_key++;
 	}
 }
+#endif
 
 /* CRC64 Test */
 ATF_TC(isc_crc64);
@@ -1847,23 +1854,128 @@ ATF_TC_BODY(isc_crc64, tc) {
 	}
 }
 
+ATF_TC(isc_hash_function);
+ATF_TC_HEAD(isc_hash_function, tc) {
+	atf_tc_set_md_var(tc, "descr", "Hash function test");
+}
+ATF_TC_BODY(isc_hash_function, tc) {
+	unsigned int h1;
+	unsigned int h2;
+
+	UNUSED(tc);
+
+	/* Incremental hashing */
+
+	h1 = isc_hash_function(NULL, 0, ISC_TRUE, NULL);
+	h1 = isc_hash_function("This ", 5, ISC_TRUE, &h1);
+	h1 = isc_hash_function("is ", 3, ISC_TRUE, &h1);
+	h1 = isc_hash_function("a long test", 12, ISC_TRUE, &h1);
+
+	h2 = isc_hash_function("This is a long test", 20,
+			       ISC_TRUE, NULL);
+
+	ATF_CHECK_EQ(h1, h2);
+
+	/* Immutability of hash function */
+	h1 = isc_hash_function(NULL, 0, ISC_TRUE, NULL);
+	h2 = isc_hash_function(NULL, 0, ISC_TRUE, NULL);
+
+	ATF_CHECK_EQ(h1, h2);
+
+	/* Hash function characteristics */
+	h1 = isc_hash_function("Hello world", 12, ISC_TRUE, NULL);
+	h2 = isc_hash_function("Hello world", 12, ISC_TRUE, NULL);
+
+	ATF_CHECK_EQ(h1, h2);
+
+	/* Case */
+	h1 = isc_hash_function("Hello world", 12, ISC_FALSE, NULL);
+	h2 = isc_hash_function("heLLo WorLd", 12, ISC_FALSE, NULL);
+
+	ATF_CHECK_EQ(h1, h2);
+
+	/* Unequal */
+	h1 = isc_hash_function("Hello world", 12, ISC_TRUE, NULL);
+	h2 = isc_hash_function("heLLo WorLd", 12, ISC_TRUE, NULL);
+
+	ATF_CHECK(h1 != h2);
+}
+
+
+ATF_TC(isc_hash_function_reverse);
+ATF_TC_HEAD(isc_hash_function_reverse, tc) {
+	atf_tc_set_md_var(tc, "descr", "Reverse hash function test");
+}
+ATF_TC_BODY(isc_hash_function_reverse, tc) {
+	unsigned int h1;
+	unsigned int h2;
+
+	UNUSED(tc);
+
+	/* Incremental hashing */
+
+	h1 = isc_hash_function_reverse(NULL, 0, ISC_TRUE, NULL);
+	h1 = isc_hash_function_reverse("\000", 1, ISC_TRUE, &h1);
+	h1 = isc_hash_function_reverse("\003org", 4, ISC_TRUE, &h1);
+	h1 = isc_hash_function_reverse("\007example", 8, ISC_TRUE, &h1);
+
+	h2 = isc_hash_function_reverse("\007example\003org\000", 13,
+				       ISC_TRUE, NULL);
+
+	ATF_CHECK_EQ(h1, h2);
+
+	/* Immutability of hash function */
+	h1 = isc_hash_function_reverse(NULL, 0, ISC_TRUE, NULL);
+	h2 = isc_hash_function_reverse(NULL, 0, ISC_TRUE, NULL);
+
+	ATF_CHECK_EQ(h1, h2);
+
+	/* Hash function characteristics */
+	h1 = isc_hash_function_reverse("Hello world", 12, ISC_TRUE, NULL);
+	h2 = isc_hash_function_reverse("Hello world", 12, ISC_TRUE, NULL);
+
+	ATF_CHECK_EQ(h1, h2);
+
+	/* Case */
+	h1 = isc_hash_function_reverse("Hello world", 12, ISC_FALSE, NULL);
+	h2 = isc_hash_function_reverse("heLLo WorLd", 12, ISC_FALSE, NULL);
+
+	ATF_CHECK_EQ(h1, h2);
+
+	/* Unequal */
+	h1 = isc_hash_function_reverse("Hello world", 12, ISC_TRUE, NULL);
+	h2 = isc_hash_function_reverse("heLLo WorLd", 12, ISC_TRUE, NULL);
+
+	ATF_CHECK(h1 != h2);
+}
+
 /*
  * Main
  */
 ATF_TP_ADD_TCS(tp) {
+	/*
+	 * Tests of hash functions, including isc_hash and the
+	 * various cryptographic hashes.
+	 */
+	ATF_TP_ADD_TC(tp, isc_hash_function);
+	ATF_TP_ADD_TC(tp, isc_hash_function_reverse);
+#ifndef PK11_MD5_DISABLE
 	ATF_TP_ADD_TC(tp, isc_hmacmd5);
+#endif
 	ATF_TP_ADD_TC(tp, isc_hmacsha1);
 	ATF_TP_ADD_TC(tp, isc_hmacsha224);
 	ATF_TP_ADD_TC(tp, isc_hmacsha256);
 	ATF_TP_ADD_TC(tp, isc_hmacsha384);
 	ATF_TP_ADD_TC(tp, isc_hmacsha512);
+#ifndef PK11_MD5_DISABLE
 	ATF_TP_ADD_TC(tp, isc_md5);
+#endif
 	ATF_TP_ADD_TC(tp, isc_sha1);
 	ATF_TP_ADD_TC(tp, isc_sha224);
 	ATF_TP_ADD_TC(tp, isc_sha256);
 	ATF_TP_ADD_TC(tp, isc_sha384);
 	ATF_TP_ADD_TC(tp, isc_sha512);
 	ATF_TP_ADD_TC(tp, isc_crc64);
+
 	return (atf_no_error());
 }
-
